package com.ruoyi.ss.transactionBill.service.impl;

import com.ruoyi.common.core.domain.BaseValidator;
import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.ss.transactionBill.domain.TransactionBillQuery;
import com.ruoyi.ss.transactionBill.domain.bo.WithdrawBO;
import com.ruoyi.ss.transactionBill.domain.dto.WithdrawDTO;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillStatus;
import com.ruoyi.ss.transactionBill.domain.enums.TransactionBillType;
import com.ruoyi.ss.transactionBill.domain.vo.TransactionBillVO;
import com.ruoyi.ss.transactionBill.service.TransactionBillService;
import com.ruoyi.ss.transactionBill.service.WithdrawValidator;
import com.ruoyi.ss.user.domain.SmUserVo;
import com.ruoyi.system.domain.enums.config.ConfigKey;
import com.ruoyi.system.service.ISysConfigService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDateTime;

/**
 * @author wjh
 * 2024/9/21
 */
@Service
public class WithdrawValidatorImpl extends BaseValidator implements WithdrawValidator {

    @Autowired
    private TransactionBillService transactionBillService;

    @Autowired
    private ISysConfigService sysConfigService;
    @Autowired
    private RedisCache redisCache;

    /**
     * 判断提现是否有风险
     *
     * @param bo
     */
    @Override
    public boolean hasRisk(WithdrawBO bo) {
        if (bo == null || bo.getUser() == null || bo.getDto() == null) {
            return false;
        }
        SmUserVo user = bo.getUser();
        WithdrawDTO dto = bo.getDto();

        // 查询最近一次的订单
        TransactionBillQuery query = new TransactionBillQuery();
        query.setUserId(user.getUserId());
        query.setType(TransactionBillType.RECHARGE.getType());
        query.setStatus(TransactionBillStatus.SUCCESS.getStatus());
        TransactionBillVO recharge = transactionBillService.selectLastOne(query);
        if (recharge == null || recharge.getPayTime() == null) {
            return false;
        }

        // 判断订单金额是否和提现一致、订单时间与提现时间是否相差较近
        // 金额一致
        boolean equalsMoney = recharge.getArrivalAmount() != null && dto.getMoney() != null && recharge.getArrivalAmount().compareTo(dto.getMoney()) == 0;
        // 提现时间相近
        int riskWithdrawTime = sysConfigService.getInt(ConfigKey.RISK_WITHDRAW_TIME); // 最低允许相隔时长（分钟）
        Duration between = Duration.between(DateUtils.toLocalDateTime(recharge.getPayTime()), LocalDateTime.now());
        boolean timeLimit = between.toMinutes() < riskWithdrawTime;
        if (equalsMoney && timeLimit) {
            return true;
        }

        return false;
    }
}
