package com.ruoyi.web.controller.ss;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.ss.realName.domain.RealName;
import com.ruoyi.ss.realName.domain.RealNameVO;
import com.ruoyi.ss.realName.domain.RealNameQuery;
import com.ruoyi.ss.realName.service.IRealNameService;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 实名认证Controller
 *
 * @author ruoyi
 * @date 2024-09-23
 */
@RestController
@RequestMapping("/ss/realName")
public class RealNameController extends BaseController
{
    @Autowired
    private IRealNameService realNameService;

    /**
     * 查询实名认证列表
     */
    @PreAuthorize("@ss.hasPermi('ss:realName:list')")
    @GetMapping("/list")
    public TableDataInfo list(RealNameQuery query)
    {
        startPage();
        startOrderBy();
        List<RealNameVO> list = realNameService.selectRealNameList(query);
        return getDataTable(list);
    }

    /**
     * 导出实名认证列表
     */
    @PreAuthorize("@ss.hasPermi('ss:realName:export')")
    @Log(title = "实名认证", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, RealNameQuery query)
    {
        List<RealNameVO> list = realNameService.selectRealNameList(query);
        ExcelUtil<RealNameVO> util = new ExcelUtil<RealNameVO>(RealNameVO.class);
        util.exportExcel(response, list, "实名认证数据");
    }

    /**
     * 获取实名认证详细信息
     */
    @PreAuthorize("@ss.hasPermi('ss:realName:query')")
    @GetMapping(value = "/{id}")
    public AjaxResult getInfo(@PathVariable("id") Long id)
    {
        return success(realNameService.selectRealNameById(id));
    }

    /**
     * 新增实名认证
     */
    @PreAuthorize("@ss.hasPermi('ss:realName:add')")
    @Log(title = "实名认证", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody RealName realName)
    {
        return toAjax(realNameService.insertRealName(realName));
    }

    /**
     * 修改实名认证
     */
    @PreAuthorize("@ss.hasPermi('ss:realName:edit')")
    @Log(title = "实名认证", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody RealName realName)
    {
        return toAjax(realNameService.updateRealName(realName));
    }

    /**
     * 删除实名认证
     */
    @PreAuthorize("@ss.hasPermi('ss:realName:remove')")
    @Log(title = "实名认证", businessType = BusinessType.DELETE)
	@DeleteMapping("/{ids}")
    public AjaxResult remove(@PathVariable Long[] ids)
    {
        return toAjax(realNameService.deleteRealNameByIds(ids));
    }
}
